#pragma once
#include <iostream>
#include <string>
#include <pcl/common/common_headers.h>
#include <pcl/io/pcd_io.h>
#include <pcl/console/parse.h>
#include <pcl/point_types.h>
#include "utils/pcl_point_type.h"
#include <algorithm>
#include <vector>

using namespace std;
struct gridFeature {
    int minz_devidedByResolution = 0;
    int deltaz_devidedByResolution = 0;
    int intensity = 0;
    bool labeled = false;
};

struct f_xyzil {
    float x = 0;
    float y = 0;
    float z = 0;
    float intensity = 0;
    float label = 0;
};


class featureExtraction {
public:
    featureExtraction(int resolutionRoad=3,int resolutionNonRoad=3) {
        setResolution(resolutionRoad, resolutionNonRoad);
    };
    ~featureExtraction() {};
    
    bool initializeGridFeature(PointCloudExport::Ptr);
   
    void setResolution(int resolutionRoad , int resolutionNonRoad ) {
        this->resolutionRoad = resolutionRoad;
        this->resolutionNonRoad = resolutionNonRoad;
    };
    
    void getFeature(vector<gridFeature>&);
    
private:
    void setRangeAndIndex();
    void toVolumePixelAndSort();

    void getNonRoadFeature(vector<gridFeature>&);
    bool binarySearch(int);

    int maxRangeRoad = 0;
    int minRangeRoad = 0;

    int maxRangeNonRoad = 0;
    int minRangeNonRoad = 0;

    int resolutionRoad;
    int resolutionNonRoad;

    int numOfIndexNonRoad = 0;
    int numOfIndexRoad = 0;

    float scoreHitMiss = -0.1;
    float scoreHitHit = 1;
    float scoreMissHit = -0.1;

    int maxScore = -9999;

    vector<f_xyzil> vec_labeledPoint;
    vector<f_xyzil> vec_unlabeledPoint;
    vector<int> vec_labeledPointIndex;
    vector<int> vec_unlabeledPointIndex;
};

