#include "block.h"

BlockGrid::BlockGrid(const Vector2i &index, float resolution)
    :GridBase(index, resolution),
      GridManager(GridResolution24cm, resolution)
{
    setOrigin(GridBase::origin_);
}

void BlockGrid::configInfo()
{
//    blockInfo_.set_local_offset_x(GridBase::center_.x());
//    blockInfo_.set_local_offset_y(GridBase::center_.y());
    auto gridGrids = getGridMap();
    auto iter = gridGrids->begin();
    distribution_.gridDistributions.resize(pow(BlockResolution / GridManager::resolution_, 2));
    for( ; iter != gridGrids->end(); iter++){
        Grid* grid = reinterpret_cast<Grid*>(iter->second.get());
        grid->configInfo();
        if(grid->getHasGround()){
            hasGround_ = true;
            break;

        }
        GridDistribution_ distributions = grid->getGridDistribution();
        uint32_t grid_id = iter->first;
        distribution_.gridDistributions.at(grid_id) = distributions;
    }

    if(hasGround_){
        GridManager::setResolution(GridResolution6cm);
        PointCloudExport::Ptr pointcloud = getBlockPointcloud();
        reDispatch(pointcloud);
        auto gridGrids = getGridMap();
        auto iter = gridGrids->begin();
        distribution_.gridDistributions.clear();
        distribution_.gridDistributions.resize(pow(BlockResolution / GridManager::resolution_, 2));
        for( ; iter != gridGrids->end(); iter++){
            Grid* grid = reinterpret_cast<Grid*>(iter->second.get());
            grid->configInfo();
            GridDistribution_ distributions = grid->getGridDistribution();
            uint32_t grid_id = iter->first;
            distribution_.gridDistributions.at(grid_id) = distributions;
        }
    }
}

BlockInfo BlockGrid::getBlockInfo()
{
    return blockInfo_;
}

BlockDistribution_ BlockGrid::getDistributions()
{
    return distribution_;
}

PointCloudExport::Ptr BlockGrid::getBlockPointcloud()
{
    PointCloudExport::Ptr pointcloud;
    pointcloud.reset(new PointCloudExport);
    auto gridGrids = getGridMap();
    auto iter = gridGrids->begin();
    for( ; iter != gridGrids->end(); iter++){
        Grid* grid = reinterpret_cast<Grid*>(iter->second.get());
        *pointcloud += *grid->getPointCloud();
    }
    return pointcloud;
}

void BlockGrid::reDispatch(PointCloudExport::Ptr pointcloud)
{
    grids_.reset(new map<uint32_t, boost::shared_ptr<GridBase>>);
    for(auto point :pointcloud->points){
        Vector2i gridIndex = getGridIndexByPoint(point);
        Grid* grid = reinterpret_cast<Grid*>(getGrid(gridIndex).get());
        grid->addPoint(point);;
    }
}

