#include<iostream>
#include <pcl/common/common_headers.h>
#include <pcl/io/pcd_io.h>
#include <pcl/point_types.h>
#include <vector>
#include <math.h>
#include <string>

using namespace std;

struct extrac{
    int minz_devidedBy3 = 0;    //最低高度/3cm
    int deltaz_devidedBy3 = 0;  //高度差/3cm
    int intensity = 0;          //平均密度
    bool labeled = false;       //是否被label为地面
};

//读取PointExport类型点云，并转换为PointXYZI类型点云，当labeled为true时表示对标记为地面的点云操作
bool exportToXYZILabeled(const pcl::PointCloud<pcl::PointExport>::Ptr cloudMixed,pcl::PointCloud<pcl::PointXYZI>::Ptr cloud, bool labeled)
{
    if(cloudMixed==nullptr||cloudMixed->points.size()==0||cloud==nullptr){
        return false;
    }
    cloud->height=1;
    cloud->is_dense =false;
    for(size_t i=0;i<cloudMixed->points.size();i++){
        if((cloudMixed->points[i].label>19999.99&&cloudMixed->points[i].label<20000.01&&labeled==true)||
        ((cloudMixed->points[i].label<19999.99||cloudMixed->points[i].label>20000.01)&&labeled==false)){
            cloud->width+=1;
            cloud->points.resize(cloud->width);
            int j=cloud->width-1;
            cloud->points[j].x=cloudMixed->points[i].x;
            cloud->points[j].y=cloudMixed->points[i].y;
            cloud->points[j].z=cloudMixed->points[i].z;
            cloud->points[j].intensity=cloudMixed->points[i].intensity;

            //cout<<setw(16)<<cloud->points[j].x<<setw(16)<<cloud->points[j].y<<setw(16)<<cloud->points[j].z<<setw(16)<<cloud->points[j].intensity<<endl;
        }
    }
    if(cloud->points.size()==0){
        return false;
    }
    //cout<<endl;
    return true;
}

//读取按label分完类的点云，并返回点云的最低高度，高度差和反射率，当labeled为true时表示对标记为地面的点云操作
extrac extract(pcl::PointCloud<pcl::PointXYZI>::Ptr cloud,bool labeled)
{
    extrac ret;
    int n=cloud->points.size();
    if(n==0){
        return {0,0,0};
    }
    float maxz=-99999.0,minz=99999.0,sum=0;
    for(int i=0;i<n;i++)
    {
        sum+=cloud->points[i].intensity;
        maxz=max(cloud->points[i].z,maxz);
        minz=min(cloud->points[i].z,minz);
    }
    //cout<<setw(16)<<maxz<<setw(16)<<minz<<endl;
    ret.intensity=round(sum/(float)n);
    //以下已转换为厘米 label的地面高度最小值取四舍五入，高度差取小，unlabel部分高度最小值取小，高度差取大
    if(labeled){

        ret.minz_devidedBy3=round(minz*100.0/3.0);
        ret.deltaz_devidedBy3=floor((maxz-minz)*100.0/3.0);
        if(ret.deltaz_devidedBy3==0){
            ret.deltaz_devidedBy3=1;
        }
        ret.labeled=true;

    }else{
        ret.minz_devidedBy3=floor(minz*100.0/3.0)-1;
        ret.deltaz_devidedBy3=ceil((maxz-minz)*100.0/3.0);
        ret.labeled=false;
    }
    return ret;
}

//此为函数的入口，输入为PointExport点云指针，输出为extrac结构体类型的vector，结构体前两项为最低高度和高度差，单位为(3厘米)；第三项为反射率，为整数；第四项为bool型，true代表该分部被地面label
vector<extrac> extractFromBlock (pcl::PointCloud<pcl::PointExport>::Ptr cloudMixed){
    vector<extrac> ret;
    if(!cloudMixed||cloudMixed->points.size()==0){
        return ret;
    }

    pcl::PointCloud<pcl::PointXYZI>::Ptr cloudLabeled(new pcl::PointCloud<pcl::PointXYZI>);
    pcl::PointCloud<pcl::PointXYZI>::Ptr cloudUnlabeled(new pcl::PointCloud<pcl::PointXYZI>);

    if(exportToXYZILabeled(cloudMixed,cloudLabeled,true)){
        ret.push_back(extract(cloudLabeled,true));
    }

    if(exportToXYZILabeled(cloudMixed,cloudUnlabeled,false)){
        ret.push_back(extract(cloudUnlabeled,false));
    };

    return ret;
}
