#ifndef GRIDMANAGER_H
#define GRIDMANAGER_H

#include "grid.h"
#include "utils/pcl_point_type.h"

class BlockGrid;

class GridManager
{
public:
    GridManager(const float resolution,
                const float sideLength);

    bool getGrid(const Vector2i &index,
                 boost::shared_ptr<GridBase> &grid);

    boost::shared_ptr<GridBase> getGrid(
            const Vector2i &index, const GridType type = gridType);

    void addGridByIndex(
            const Vector2i &index, const GridType type = gridType);

    Vector2i getGridIndexByPoint(const PointExport &point);

    void setOrigin(const Vector2f &origin);

    const boost::shared_ptr<map<uint32_t, boost::shared_ptr<GridBase>>>
    getGridMap();

    void setResolution(float resolution);

    void setSideLength(float sideLength);

    float getResolution();
protected:
    uint32_t encodeIndex(const Vector2i &index);
protected:
    Vector2f origin_ = Vector2f::Zero();

    boost::shared_ptr<map<uint32_t, boost::shared_ptr<GridBase>>> grids_;

    float resolution_;

    float sideLength_;

    uint32_t itemCntPerSide_ = 0;
private:


    boost::shared_mutex insertMtx_;

};

#endif // GRIDMANAGER_H
