#include "blockserializer.h"
#include "proto_pb/common/header.pb.h"
#include "proto_pb/block/block_index.pb.h"
#include "proto_pb/block/block_info.pb.h"
#include "utils/basic_functions.h"

BlockSerializer::BlockSerializer(
        boost::shared_ptr<GridManager> blockManager)
    :blockManager_(blockManager)
{

}

bool checkOneDistribution_(
    const OneDistribution_ &source,
    const OneDistribution_ &target){
    if(source.floor == target.floor
            && source.z_delta_level == target.z_delta_level
            && source.intensity == target.intensity){
        return true;
    }else{
        return false;
    }
}

bool checkGridDistribution_(
        const GridDistribution_ &source,
        const GridDistribution_ &target){
    if(source.oneDistributions.size()
            != target.oneDistributions.size()){
        return false;
    }
    for(size_t i = 0; i < source.oneDistributions.size(); i++){
        if(!checkOneDistribution_(
                    source.oneDistributions[i],
                    target.oneDistributions[i])){
            return false;
        }
    }
    return true;
}

struct DistributionInfo{
    GridDistribution_ gridDistribution;
    uint32_t cnt = 0;
    uint32_t id = 0;
};

void BlockSerializer::configSerialization()
{
    string serializedStream;

    auto blockGrids = blockManager_->getGridMap();

//    auto it = blockGrids->begin();
//    for(int temp = 0; temp < 50; temp++){
//        it++;
//    }
//    auto block = *it;
//    blockGrids->clear();
//    blockGrids->insert(make_pair(block.first, block.second));

    vector<BlockInfo_> blockInfoVec;

    vector<DistributionInfo> distributionInfoVec;

    LOG(INFO) << "blockGrids->size(): " << blockGrids->size();
    map<int64_t, DistributionInfo> distributionMap;
    for(auto iter = blockGrids->begin(); iter != blockGrids->end(); iter++){
        BlockGrid* block = reinterpret_cast<BlockGrid*>(iter->second.get());
        int32_t blockId = iter->first;
        BlockDistribution_ blockDistribution =
                block->getDistributions();
        for(size_t grid_id = 0; grid_id < blockDistribution.gridDistributions.size(); grid_id++){
            GridDistribution_ gridDistribution = blockDistribution.gridDistributions[grid_id];
            int64_t key = gridDistribution.getScore();
            if(key == 0){
                continue;
            }
            auto iter = distributionMap.find(key);
            if(iter == distributionMap.end()){
                DistributionInfo distributionInfo;
                distributionInfo.gridDistribution = gridDistribution;
//                distributionInfo.id = distributionInfoVec.size();
//                distributionInfoVec.push_back(distributionInfo);
                distributionMap.insert(make_pair(key, distributionInfo));
            }else{
                iter->second.cnt++;
            }
        }

        BlockInfo_ blockInfo_;
        blockInfo_.block_id = blockId;
        blockInfo_.grid_resolution = block->GridManager::getResolution();
        blockInfo_.bit_map.resize(pow(BlockResolution / block->GridManager::getResolution(), 2) / 8);
//        blockInfo_.grid_feature_index.resize(90000, 0);
        blockInfoVec.push_back(blockInfo_);
    }


////    vector<DistributionInfo> distributionInfoVec;
///
    for(auto iter = distributionMap.begin(); iter != distributionMap.end(); iter++){
        distributionInfoVec.push_back(iter->second);
    }
    sort(distributionInfoVec.begin(),
         distributionInfoVec.end(),
         [](const DistributionInfo& query, const DistributionInfo& target) {
             return query.cnt > target.cnt;
         });

    map<int64_t, DistributionInfo>().swap(distributionMap);
    for(size_t i = 0; i < distributionInfoVec.size(); i++){
        auto distributionInfo = distributionInfoVec[i];
        distributionInfo.id = i;
        distributionMap.insert(make_pair(distributionInfo.gridDistribution.getScore(), distributionInfo));
    }

    LOG(INFO) << "distributionInfoVec.size(): " << distributionInfoVec.size();
    size_t index = 0;
    for(auto iter = blockGrids->begin(); iter != blockGrids->end(); iter++){
        BlockGrid* block = reinterpret_cast<BlockGrid*>(iter->second.get());
        BlockDistribution_ blockDistribution =
                block->getDistributions();
        uint8_t byte = 0;
        for(size_t grid_id = 0; grid_id < blockDistribution.gridDistributions.size(); grid_id++){
            GridDistribution_ gridDistribution = blockDistribution.gridDistributions[grid_id];
            uint32_t byteIndex = grid_id / 8;
            uint8_t bitIndex = grid_id % 8;
            int64_t key = gridDistribution.getScore();
            auto iter = distributionMap.find(key);
            if(iter != distributionMap.end()){
                size_t distributionId = iter->second.id;
                byte |= 1 << bitIndex;
                blockInfoVec[index].active_grid_feature_index.push_back(distributionId);
                blockInfoVec[index].whole_grid_feature_index.push_back(distributionId + 1);
            }else{
                blockInfoVec[index].whole_grid_feature_index.push_back(0);
            }
            if(bitIndex == 7){
                blockInfoVec[index].bit_map[byteIndex] = byte;
                byte = 0;
            }
        }
        blockInfoVec[index].bit_map.back() = byte;
        index++;
    }

    /////////// BlockIndex /////////////
    /// \brief blockIndex
    ///

    BlockIndex blockIndex;

    /////////// BlockInfo /////////////
    /// \brief blockInfo
    ///

    string blockInfoBitmapStream, blockInfoIndexStream;
    uint32_t block_offset = 0;


    LOG(INFO) << "blockInfoVec.size(): " << blockInfoVec.size();
    int cnt = 0;
    for(auto &blockInfo_ : blockInfoVec){
        BlockInfo blockInfoBitmap;
        for(auto byte : blockInfo_.bit_map){
            blockInfoBitmap.add_bit_map(byte);
        }
        for(auto gridIndex : blockInfo_.active_grid_feature_index){
            blockInfoBitmap.add_grid_feature_index(gridIndex);
        }
        cnt++;

        blockInfoBitmap.set_grid_resolution(blockInfo_.grid_resolution);
        blockInfoBitmapStream += blockInfoBitmap.SerializeAsString();
        BlockDescribe* blockDescribe = blockIndex.add_block_describe();
        blockDescribe->set_block_id(blockInfo_.block_id);
        blockDescribe->set_block_info_offset(block_offset);
        block_offset = blockInfoBitmapStream.size();


        BlockInfo blockInfoIndex;
        for(auto gridIndex : blockInfo_.whole_grid_feature_index){
            blockInfoIndex.add_grid_feature_index(gridIndex);
        }
        blockInfoIndexStream += blockInfoIndex.SerializeAsString();
    }
    LOG(INFO) << "cnt: " << cnt;

    Distributions distributions;
    for(auto &distributionInfo : distributionInfoVec){
        GridDistribution* gridDistribution = distributions.add_grid_distribution();
        for(auto &oneDistribution : distributionInfo.gridDistribution.oneDistributions){
            OneDistribution* protoOneDistribution = gridDistribution->add_one_distribution();
            protoOneDistribution->set_floor(oneDistribution.floor);
            protoOneDistribution->set_z_delta_leval(oneDistribution.z_delta_level);
            protoOneDistribution->set_intensity(oneDistribution.intensity);
        }
    }

    /////////// Header /////////////
    /// \brief header
    ///
    Header header;
    header.set_save_type(Header_SaveType_DIRECT);
    header.set_mesh_level(15);
    header.set_block_relolution(BlockResolution);
    header.set_grid_relolution(GridResolution6cm);
    header.set_offset_x(offset_.x());
    header.set_offset_y(offset_.y());
    header.set_longtitude(121.131722586496);
    header.set_latitude(31.1529105050223);
    header.set_height(0);

    LOG(INFO) << "blockInfoBitmapStream.size(): " << blockInfoBitmapStream.size();
    LOG(INFO) << "blockInfoIndexStream.size(): " << blockInfoIndexStream.size();

    string blockInfoStream;
    if(blockInfoBitmapStream.size() <  blockInfoIndexStream.size()){
        blockInfoStream = blockInfoBitmapStream;
        header.set_save_type(Header_SaveType_BIT);
        LOG(INFO) << "Header_SaveType_BIT";
    }else{
        blockInfoStream = blockInfoIndexStream;
        header.set_save_type(Header_SaveType_DIRECT);
        LOG(INFO) << "Header_SaveType_DIRECT";
    }

    header.set_active_block_count(blockGrids->size());

    LOG(INFO) << "blockIndex.block_describe_size(): " << blockIndex.block_describe_size();

    string blockIndexStream = blockIndex.SerializeAsString();
    string distributionsStream = distributions.SerializeAsString();

    header.set_block_index_length(blockIndexStream.size());
    header.set_block_info_length(blockInfoBitmapStream.size());

    string headerStream = header.SerializeAsString();

    serializedStream += headerStream;
    LOG(INFO) << "headerStream.size(): " << headerStream.size();
    serializedStream += blockIndexStream;
    LOG(INFO) << "blockIndexStream.size(): " << blockIndexStream.size();
    serializedStream += blockInfoStream;
    LOG(INFO) << "blockInfoStream.size(): " << blockInfoStream.size();
    serializedStream += distributionsStream;
    LOG(INFO) << "distributionsStream.size(): " << distributionsStream.size();

    uint16_t headerSize = headerStream.size();

    string path = "/home/juefx/temp.stream";
    std::ofstream ofs(path, ios::binary);
    if(!ofs){
        LOG(WARNING) << path << " load fail!";
        return;
    }
    ofs << (char)headerSize << (char)(headerSize >> 8);
    ofs << serializedStream;
\
    ofs.close();
}

void BlockSerializer::setOffset(const Vector3f &offset)
{
    offset_ = offset;
}
